@@ -32,12 +32,15 @@ from translate.lang import data
 from translate.misc.multistring import multistring
 from translate.storage import base, lisa
 
-EOF = None
 WHITESPACE = {" ", "\n", "\t"}  # Whitespace that we collapse.
 ESCAPE_NEWLINE = {"n", "N"}
 ESCAPE_TAB = {"t", "T"}
-ESCAPE_PLAIN = {" ", '"', "'", "@", "?"}
-MULTIWHITESPACE = re.compile("[ \n\t]{2}(?!\\\\n)")
+ESCAPE_PLAIN = {" ", '"', "'", "@", "?", "\\"}
+MULTIWHITESPACE = re.compile(r"[ \n\t]{2}(?!\\n)")
+QUOTED_STRING = re.compile(r'((?<!\\)"(?:\\"|[^"])*(?<!\\)")')
+UNICODE_ESCAPE = re.compile(r"\\u([a-fA-F0-9]{4})")
+CHAR_ESCAPE = re.compile(r"\\(.)")
+WHITESPACE_RE = re.compile(r"\s+")
 
 ESCAPE_TRANSLATE = str.maketrans(
     {
@@ -70,6 +73,7 @@ class DecodingXMLParser:
         self.character_data = False
         self.raw_string = True
         self.in_string = False
+        self.inside_quoted = False
         self.do_cleanup = False
         self.depth = 0
         self.parser = parser = ParserCreate()
@@ -84,135 +88,81 @@ class DecodingXMLParser:
         parser.DefaultHandler = self.DefaultHandler
 
     @staticmethod
-    def process_string(text: str) -> tuple[str, bool, bool]:
-        """
-        Remove escaping from Android resource.
+    def decode_unicode_escapes(match: re.Match) -> str:
+        escaped = match.group(1)
+        # Convert from hex to character
+        return chr(int(escaped, 16))
 
-        Code is based on android2po
-        <https://github.com/miracle2k/android2po>
-        """
-        # Mirror globals into locals for faster lookups
-        eof = EOF
-        whitespace = WHITESPACE
-        escape_newline = ESCAPE_NEWLINE
-        escape_tab = ESCAPE_TAB
-        escape_plain = ESCAPE_PLAIN
-        cleanup_start = True
-        cleanup_end = True
+    @staticmethod
+    def decode_escapes(match: re.Match) -> str:
+        escaped = match.group(1)
+        if escaped in ESCAPE_NEWLINE:
+            return "\n"  # an actual newline
+        if escaped in ESCAPE_TAB:
+            return "\t"  # an actual tab
+        if escaped in ESCAPE_PLAIN:
+            # Plain string to escape
+            return escaped
+        # All others, remove, like Android does as well.
+        return ""
 
-        # We need to collapse multiple whitespace while paying
-        # attention to Android's quoting and escaping.
-        space_count = 0
-        active_quote = False
-        active_escape = False
-        i = 0
-        text = [*text, eof]
-        while i < len(text):
-            c = text[i]
+    def strip_part(self, text: str) -> tuple[str, bool]:
+        # Detect leading quote
+        if text.startswith('"'):
+            text = text[1:]
+            self.inside_quoted ^= True
 
-            if not active_escape:
-                # Handle whitespace collapsing
-                if c is not eof and c in whitespace:
-                    space_count += 1
-                elif space_count >= 1:
-                    # Remove sequential whitespace; Pay attention: We
-                    # don't do this if we are currently inside a quote,
-                    # except for one special case: If we have unbalanced
-                    # quotes, e.g. we reach eof while a quote is still
-                    # open, we *do* collapse that trailing part; this is
-                    # how Android does it, for some reason.
-                    if not active_quote or c is eof:
-                        # Replace by a single space, will get rid of
-                        # non-significant newlines/tabs etc.
-                        text[i - space_count : i] = " "
-                        i -= space_count - 1
-                    space_count = 0
-                else:
-                    space_count = 0
+        # Check if whitespace should be stripped
+        strip_whitespace = not self.inside_quoted
 
-            # Handle quotes
-            if c == '"' and not active_escape:
-                if i == 0:
-                    cleanup_start = False
-                if active_quote:
-                    cleanup_end = False
-                active_quote = not active_quote
-                del text[i]
-                i -= 1
-            elif c is not eof:
-                cleanup_end = True
+        # Detect trailing quote
+        if text.endswith('"') and not text.endswith('\\"'):
+            text = text[:-1]
+            self.inside_quoted ^= True
 
-            # Handle escapes
-            if c == "\\":
-                if i == 0:
-                    cleanup_start = False
-                if not active_escape:
-                    active_escape = True
-                else:
-                    # A double-backslash represents a single;
-                    # simply deleting the current char will do.
-                    del text[i]
-                    i -= 1
-                    active_escape = False
-            elif active_escape:
-                # Handle the limited amount of escape codes
-                # that we support.
-                # TODO: What about \r, or \r\n?
-                cleanup_end = False
-                if c is eof:
-                    # Basically like any other char, but put
-                    # this first so we can use the ``in`` operator
-                    # in the clauses below without issue.
-                    pass
-                elif c in escape_newline:
-                    text[i - 1 : i + 1] = "\n"  # an actual newline
-                    i -= 1
-                elif c in escape_tab:
-                    text[i - 1 : i + 1] = "\t"  # an actual tab
-                    i -= 1
-                elif c in escape_plain:
-                    text[i - 1 : i] = ""  # remove the backslash
-                    i -= 1
-                elif c == "u":
-                    # Unicode sequence. Android is nice enough to deal
-                    # with those in a way which let's us just capture
-                    # the next 4 characters and raise an error if they
-                    # are not valid (rather than having to use a new
-                    # state to parse the unicode sequence).
-                    # Exception: In case we are at the end of the
-                    # string, we support incomplete sequences by
-                    # prefixing the missing digits with zeros.
-                    # Note: max(len()) is needed in the slice due to
-                    # trailing ``None`` element.
-                    max_slice = min(i + 5, len(text) - 1)
-                    codepoint_str = "".join(text[i + 1 : max_slice])
-                    if len(codepoint_str) < 4:
-                        codepoint_str = "0" * (4 - len(codepoint_str)) + codepoint_str
-                    # We can't trust int() to raise a ValueError,
-                    # it will ignore leading/trailing whitespace.
-                    if not codepoint_str.isalnum():
-                        raise ValueError(
-                            f"Invalid unicode escape sequence: {codepoint_str!r}"
-                        )
-                    try:
-                        codepoint = chr(int(codepoint_str, 16))
-                    except ValueError:
-                        raise ValueError(
-                            f"Invalid unicode escape sequence: {codepoint_str!r}"
-                        )
+        # Consolidate whitespace if needed
+        if strip_whitespace:
+            text = WHITESPACE_RE.sub(" ", text)
 
-                    text[i - 1 : max_slice] = codepoint
-                    i -= 1
-                else:
-                    # All others, remove, like Android does as well.
-                    text[i - 1 : i + 1] = ""
-                    i -= 1
-                active_escape = False
+        return text, strip_whitespace
 
-            i += 1
+    def process_string(self, content: str) -> tuple[str, bool, bool]:
+        # Skip processing for a blank string
+        if not content:
+            return "", False, False
 
-        # Join the string together again, but w/o EOF marker
-        return "".join(text[:-1]), cleanup_start, cleanup_end
+        # Extract quoted parts, remove blank ones
+        parts = [part for part in QUOTED_STRING.split(content) if part]
+
+        # Detect possible cleanups
+        cleanup_start = False
+        cleanup_end = False
+
+        # Collapse whitespace
+        stripped_parts: list[str] = []
+        for i, part in enumerate(parts):
+            stripped, was_stripped = self.strip_part(part)
+            if i == 0:
+                cleanup_start = was_stripped
+            cleanup_end = was_stripped and not self.inside_quoted
+            stripped_parts.append(stripped)
+
+        # Merge this back to a single string
+        text = "".join(stripped_parts)
+
+        # Handle unicode escapes
+        unescaped_text = UNICODE_ESCAPE.sub(self.decode_unicode_escapes, text)
+        # Handle other escapes
+        unescaped_text = CHAR_ESCAPE.sub(self.decode_escapes, unescaped_text)
+
+        # Detect escaping at the start and end of a string (we actually care only for
+        # whitespace escapes, but detecting more does not matter)
+        if text and unescaped_text and text[0] != unescaped_text[0]:
+            cleanup_start = False
+        if text and unescaped_text and text[-1] != unescaped_text[-1]:
+            cleanup_end = False
+
+        return unescaped_text, cleanup_start, cleanup_end
 
     def cleanup_text(self, text: str) -> str:
         """Remove leading whitespace from text."""
@@ -282,8 +232,7 @@ class EncodingXMLParser(DecodingXMLParser):
     EMIT_DEPTH = 0
     FOREGIN_DTD = False
 
-    @staticmethod
-    def process_string(text: str) -> tuple[str, bool, bool]:
+    def process_string(self, text: str) -> tuple[str, bool, bool]:
         return (
             AndroidResourceUnit.escape(text, quote_wrapping_whitespaces=False),
             False,
@@ -388,22 +337,21 @@ class AndroidResourceUnit(base.TranslationUnit):
 
     def get_xml_text_value(self, xmltarget):
         raw_xml = etree.tostring(xmltarget, encoding="unicode", with_tail=False)
+        # Use decoding parser to keep XML entitities, CDATA while processing Android escaping
+        parser = DecodingXMLParser(raw_xml)
+
         # Unescape as plain text in case there is no XML markup. Ufortunately
         # CDATA elements are not listed as childs in lxml, so test for it in raw XML.
         if len(xmltarget) == 0 and "<![CDATA[" not in raw_xml:
             if xmltarget.text is None:
                 return ""
-            text, cleanup_start, cleanup_end = DecodingXMLParser.process_string(
-                xmltarget.text
-            )
+            text, cleanup_start, cleanup_end = parser.process_string(xmltarget.text)
             if cleanup_start:
                 text = text.lstrip()
             if cleanup_end:
                 text = text.rstrip()
             return text
 
-        # Use decoding parser to keep XML entitities, CDATA while processing Android escaping
-        parser = DecodingXMLParser(raw_xml)
         return parser.parse()
 
     def set_xml_text_plain(self, target, xmltarget):
@@ -579,7 +527,7 @@ class AndroidResourceFile(lisa.LISAfile):
     bodyNode = "resources"
     XMLskeleton = """<?xml version="1.0" encoding="utf-8"?>
 <resources></resources>"""
-    XMLindent = {"indent": "    ", "leaves": {"string", "item"}}
+    XMLindent = {"indent": "    ", "leaves": {"string", "item"}, "trailing_eol": True}
     XMLuppercaseEncoding = False
 
     def initbody(self):
